#!/bin/sh
# Batch/Script file for convenient execution of ZipSnapNG
# Instructions:
# 1. Change "ZipSnapDir" to the full path of the ZipSnapNG base directory (where the README file is located)
# 2. Optionally change "JavaFullPath" to the full path of the java executable if not in the PATH
# 3. Optionally adjust "MaxMemoryMb" to set the maximum amount of memory the JVM can use
# 4. Optionally copy this file to a directory on the PATH
# 5. Make both ZipSnapNG.sh and ZipSnapNG-GUI.sh executable (e.g. chmod 755)

# Configuration settings
ZipSnapDir="/usr/local/lib/ZipSnapNG"
JavaFullPath="java"
MaxMemoryMb="512"

# Do not change anything below this line
ZipSnapVersion="3.1.0"
Zip4jVersion="1.3.2-1"
PropertiesFile="${ZipSnapDir}/etc/ZipSnapNG.properties"
FileEncoding="UTF-8"
ClassPath="${ZipSnapDir}/lib/ZipSnapNG-${ZipSnapVersion}.jar:${ZipSnapDir}/lib/zip4j-${Zip4jVersion}.jar"
# JavaOptions="-javaagent:${ZipSnapDir}/lib/classmexer.jar"

if [ ! -e "$ZipSnapDir" ]
then
  echo "ERROR: the ZipSnapDir directory $ZipSnapDir does not exist. Please review the installation instructions"
  exit 1
fi
if [ ! -e "${ZipSnapDir}/lib/zip4j-${Zip4jVersion}.jar" ]
then
  echo "ERROR: the Zip4j library cannot be found at ${ZipSnapDir}/lib/zip4j-${Zip4jVersion$}.jar"
  exit 1
fi
if [ ! -e "${ZipSnapDir}/lib/ZipSnapNG-${ZipSnapVersion}.jar" ]
then
  echo "ERROR: the ZipSnapNG jar cannot be found at ${ZipSnapDir}/lib/ZipSnapNG-${ZipSnapVersion}.jar"
  exit 1
fi

Cmd="${JavaFullPath} -Xmx${MaxMemoryMb}m -classpath ${ClassPath} -Dfile.encoding=${FileEncoding} -DpropertiesFile=${PropertiesFile} ${JavaOptions} com.backup.zipsnap.ng.Main $@"
#echo $Cmd
eval "$Cmd"
