package com.backup.zipsnap.ng.gui;

import java.awt.EventQueue;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import com.backup.zipsnap.ng.Main;
import com.backup.zipsnap.ng.gui.util.DialogFrame;
import com.backup.zipsnap.ng.gui.util.FileChooser;
import com.backup.zipsnap.ng.gui.util.WindowsManager;
import com.backup.zipsnap.ng.job.util.FileIO;
import com.javamex.classmexer.MemoryUtil;
import com.javamex.classmexer.MemoryUtil.VisibilityFilter;

import java.awt.Toolkit;

public class GUI {

	public boolean isInGUI;
	public JFrame mainWindow;
	public JTabbedPane tabbedPanel;
	public MenuBar menuBar;
	public WindowsManager windowsManager;
	public SettingsFrame settingsFrame;

	/*
	 * run the application in the a new thread
	 */
	public void show() {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					/* create the GUI and show it */
					GUI gui = new GUI(isInGUI);
					gui.mainWindow.setVisible(true);
					/* restore opened tabs */
					if (Main.settings.restoreOpenedTabs && Main.settings.openedTabs.size() > 0) {
						for (String file : Main.settings.openedTabs) {
							GUI.print("Restoring tab "+file);
							gui.addTab(new File(file),Tab.action.RESTORE);
						}
					} else gui.windowsManager.show(new DialogFrame(DialogFrame.type.INFO,"Create a new backup job or load an existing one from the File menu to begin"));
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/*
	 * create the application
	 */
	public GUI(boolean isInGUI) {
		this.isInGUI = isInGUI;
		/* if running from CLI do nothing */
		if (!isInGUI) return;
		
		/* draw the main window */
		mainWindow = new JFrame();
		mainWindow.setIconImage(Toolkit.getDefaultToolkit().getImage(GUI.class.getResource("/com/sun/java/swing/plaf/windows/icons/Computer.gif")));
		mainWindow.setResizable(false);
		mainWindow.setTitle(Main.programTitle);
		mainWindow.setBounds(100, 100, 1024, 768);
		mainWindow.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		mainWindow.addWindowListener( new WindowAdapter() {
            public void windowClosing(WindowEvent we) {
            	exit();
            }
        } );
		
		/* instantiate the window manager */
		windowsManager = new WindowsManager(this);
		/* load the settings frame */
		settingsFrame = new SettingsFrame(this);

		/* draw the menu */
		menuBar = new MenuBar(this);
		mainWindow.setJMenuBar(menuBar);
		
		/* draw the tabbed panel */		
		JPanel body = new JPanel();
		body.setLayout(null);
		tabbedPanel = new JTabbedPane(JTabbedPane.TOP);
		tabbedPanel.setBounds(5, 0, 1008, 768);
		body.add(tabbedPanel);
		mainWindow.getContentPane().add(body);
	}
	
	/*
	 * return the selected tab
	 */
	public Tab getSelectedTab() {
		return (Tab)tabbedPanel.getSelectedComponent();
	}
	
	/*
	 * return the jobPanel of the selected tab
	 */
	public JobPanel getSelectedTabPanel() {
		return getSelectedTab().jobPanel;
	}	
	
	/*
	 * print messages to standard output
	 */
	public static void print(String message) {
		if (Main.settings.guiDebug) System.out.println(message);
	}
	
	/*
	 * calculate memory usage of a given object 
	 */
	public static void memoryUsage(String name, Object o) {
		if (Main.settings.memoryStats) 
			GUI.print("(MEMORY-STATS) "+name+": " + FileIO.getSize(MemoryUtil.deepMemoryUsageOf(o,VisibilityFilter.ALL)));
	}

	
	/*
	 * add a new tab to the panel
	 */
	public void addTab(File configFile, Tab.action action) {
		/* make the user choosing a file */
		if (action == Tab.action.CREATE) {
			String filename = FileChooser.open(mainWindow, "Create a new config file",FileChooser.action.SAVE, FileChooser.filter.JOBS);
			if (!filename.equals("")) configFile = new File(filename+".job");
		}
		if (action == Tab.action.LOAD) {
			String filename = FileChooser.open(mainWindow, "Select an existing config file",FileChooser.action.OPEN, FileChooser.filter.JOBS);
			if (!filename.equals("")) configFile = new File(filename);
		}

		if (configFile != null) {
			Tab jobTab = new Tab(this,configFile);
			tabbedPanel.add(jobTab);
			if (action == Tab.action.RESTORE && Main.settings.restoreOpenedTabs && Main.settings.openedTabsSelectedIndex < tabbedPanel.getTabCount()) 
				tabbedPanel.setSelectedIndex(Main.settings.openedTabsSelectedIndex);
			else tabbedPanel.setSelectedComponent(jobTab); 
			if (action == Tab.action.CREATE) jobTab.actionCreateJob();
			if (action == Tab.action.LOAD) jobTab.actionLoadJob();
			if (action == Tab.action.RESTORE) jobTab.actionLoadJob();
		}
	}

	/*
	 * close the current tab
	 */
	public void closeTab(Tab tab) {
		final Tab selectedTab = tab == null ? (Tab)tabbedPanel.getSelectedComponent() : tab;
		if (selectedTab.jobEngine.isRunning) {
			/* if a job is running ask if it has to be stopped */
			DialogFrame question = (DialogFrame) windowsManager.show(new DialogFrame(DialogFrame.type.YES_NO,"A job is still running, do you want to stop it before closing the tab?"));
			question.addPropertyChangeListener(new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent action) {
					if (action.getNewValue() == DialogFrame.action.YES) {
						selectedTab.jobPanel.actionStop();
					}
				}
			});
		} 
	}
	
	/*
	 * remove a given tab from the tabbed panel
	 */	
	public void removeTab(Tab tab) {
		Tab selectedTab = tab == null ? (Tab)tabbedPanel.getSelectedComponent() : tab;
		tabbedPanel.remove(selectedTab);
		/* if the are no more tabs, reset the menu bar */ 
		if (tabbedPanel.getTabCount() == 0) menuBar.setStatus(MenuBar.status.INIT);
	
	}
	
	/*
	 * close all the tabs
	 */
	public void closeAllTabs() {
		Main.settings.openedTabs.clear();
		Main.settings.set("openedTabsSelectedIndex", tabbedPanel.getSelectedIndex()+"");
		/* close each tab */
		for (int i = 0; i < tabbedPanel.getTabCount(); i++) {
			Tab tab = (Tab)tabbedPanel.getComponent(i);
			if (!Main.settings.openedTabs.contains(tab.configFile.getAbsolutePath())) Main.settings.openedTabs.add(tab.configFile.getAbsolutePath());
			GUI.memoryUsage("Tab "+tab.getName(),tab);
			closeTab(tab);
		}
		tabbedPanel.removeAll();
		/* save opened tabs */
		if (Main.settings.restoreOpenedTabs) {
			String openedTabsString = "";
			for (int i = 0 ; i < Main.settings.openedTabs.size(); i++) {
				openedTabsString = openedTabsString+Main.settings.openedTabs.get(i);
				if (i != Main.settings.openedTabs.size()-1) openedTabsString = openedTabsString+";";
			}
			Main.settings.set("openedTabs", openedTabsString);
			Main.settings.save();
		}
	}
	
	/*
	 * exit the application
	 */
	public void exit() {
		if (Main.settings.confirmExit) {
			DialogFrame question = (DialogFrame) windowsManager.show(new DialogFrame(DialogFrame.type.YES_NO,"Do you really want to exit?"));
			question.addPropertyChangeListener(new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent action) {
					if (Main.settings.confirmExit && action.getNewValue() == DialogFrame.action.YES) doClose();
				}
			});
		} else doClose();
	}
	
	/*
	 * close the application 
	 */
	private void doClose() {
		closeAllTabs();
		mainWindow.dispose();
		System.exit(0);
	}
	

	

}
