/**
 * TERMS AND CONDITIONS:
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.backup.zipsnap.ng.gui;

import java.awt.Desktop;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.net.URL;

import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JSeparator;

import com.backup.zipsnap.ng.Main;
import com.backup.zipsnap.ng.gui.util.DialogFrame;


public class MenuBar extends JMenuBar {

	private static final long serialVersionUID = -3630845563078531407L;
	private GUI gui;
	private JMenuItem jobNew;
	private JMenuItem jobOpen;
	private JMenuItem jobClose;
	private JMenuItem jobReload;
	private JMenuItem jobEdit;
	private JMenuItem runBackup;
	private JMenuItem runRestore;
	private JMenuItem runListFiles;
	private JMenuItem runPurge;
	private JMenuItem runStop;
	public static enum status { RUNNING,READY,INIT; }
	

	/*
	 * Create the menu panel
	 */
	public MenuBar(final GUI gui) {
		this.gui = gui;

		
		JMenu mnBlankMenu = new JMenu(" ");
		add(mnBlankMenu);
		
		JMenu file = new JMenu("File");
		add(file);
		jobNew = new JMenuItem("New Job");
		jobNew.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				DialogFrame question = (DialogFrame) gui.windowsManager.show(new DialogFrame(DialogFrame.type.INFO,"Select the location when you want to store your backup configuration file, give it a friendly name and then chose the appropriate options"));
				question.addPropertyChangeListener(new PropertyChangeListener() {
					public void propertyChange(PropertyChangeEvent action) {
						gui.addTab(null,Tab.action.CREATE);
					}
				});
			}
		});
		file.add(jobNew);
		jobOpen = new JMenuItem("Open Job");
		jobOpen.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				gui.addTab(null,Tab.action.LOAD);
			}
		});
		file.add(jobOpen);
		jobReload = new JMenuItem("Reload Job");
		jobReload.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				/* reload the current tab */
				gui.getSelectedTab().actionLoadJob();
			}
		});
		file.add(jobReload);
		jobClose = new JMenuItem("Close Job");
		jobClose.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				/* close the current tab */
				gui.closeTab(null);
				gui.removeTab(null);
			}
		});
		file.add(jobClose);
		file.add(new JSeparator());
		JMenuItem settings = new JMenuItem("Settings");
		settings.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				gui.settingsFrame.load();
			}
		});
		file.add(settings);
		file.add(new JSeparator());
		JMenuItem exit = new JMenuItem("Exit");
		exit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				/* exit the application */
				gui.exit();
			}
		});
		file.add(exit);
		
		/* job menu */
		JMenu job = new JMenu("Job");
		job.addMouseListener(new MouseAdapter() {
			public void mouseEntered(MouseEvent arg0) {
				updateStatus();
			}
		});
		add(job);
		jobEdit = new JMenuItem("Edit");
		jobEdit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				/* edit the configuration of the current tab */
				gui.getSelectedTab().actionEditJobConfig(false);
			}
		});
		job.add(jobEdit);
		job.add(new JSeparator());
		runBackup = new JMenuItem("Run backup");
		runBackup.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				/* run a backup job */
				gui.getSelectedTabPanel().actionBackup();
			}
		});
		job.add(runBackup);
		runRestore = new JMenuItem("Run restore");
		runRestore.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				/* run a restore job */
				gui.getSelectedTabPanel().actionRestore();
			}
		});
		job.add(runRestore);
		runPurge = new JMenuItem("Run purge");
		runPurge.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				/* run a purge job */
				gui.getSelectedTabPanel().actionPurge();
			}
		});
		job.add(runPurge);
		runListFiles = new JMenuItem("List Files");
		runListFiles.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				/* run a snapshot info job */
				gui.getSelectedTabPanel().actionListFiles();
			}
		});
		job.add(runListFiles);
		
		job.add(new JSeparator());
		runStop = new JMenuItem("Kill job");
		runStop.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				/* stop the current job if running */
				gui.getSelectedTabPanel().actionStop();
			}
		});
		job.add(runStop);

		/* Help menu */		
		JMenu help = new JMenu("Help");
		add(help);
		JMenuItem manual = new JMenuItem("Online Manual");
		manual.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				  Desktop desktop = Desktop.isDesktopSupported() ? Desktop.getDesktop() : null;
				    if (desktop != null && desktop.isSupported(Desktop.Action.BROWSE)) {
				    	/* open the browser at the website page */
				        try { desktop.browse(new URL(Main.website).toURI()); } 
				        catch (Exception exception) {}
				    }
			}
		});
		help.add(manual);
		JSeparator separator_1 = new JSeparator();
		help.add(separator_1);
		JMenuItem about = new JMenuItem("About");
		about.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				/* show an about dialog */
				gui.windowsManager.show(new DialogFrame(DialogFrame.type.INFO,Main.programTitle+
						"\n==================================\n\n"+
						"Simple multi-platform backup tool with advanced features"));
			}
		});
		help.add(about);
		
		setStatus(status.INIT);
	}
	
	/*
	 * update the status of the menu bar based on the job's status 
	 */
	private void updateStatus() {
		if (gui.tabbedPanel.getTabCount() > 0) { 
			if (gui.getSelectedTab().jobEngine.isRunning) setStatus(status.RUNNING);
			else setStatus(status.READY);
		}
	}
	
	/*
	 * enable/disable the menu items based on the status
	 */
	public void setStatus(status what) {
		if (what == status.INIT) {
			jobNew.setEnabled(true);
			jobOpen.setEnabled(true);
			jobClose.setEnabled(false);
			jobReload.setEnabled(false);

			jobEdit.setEnabled(false);
			runBackup.setEnabled(false);
			runRestore.setEnabled(false);
			runPurge.setEnabled(false);
			runListFiles.setEnabled(false);
			runStop.setEnabled(false);
		}
		else if (what == status.READY) {
			jobNew.setEnabled(true);
			jobOpen.setEnabled(true);
			jobClose.setEnabled(true);
			jobReload.setEnabled(true);
			
			jobEdit.setEnabled(true);
			runBackup.setEnabled(true);
			runRestore.setEnabled(true);
			runPurge.setEnabled(true);
			runListFiles.setEnabled(true);
			runStop.setEnabled(false);
		}
		else if (what == status.RUNNING) {
			jobNew.setEnabled(true);
			jobOpen.setEnabled(true);
			jobClose.setEnabled(true);
			jobReload.setEnabled(false);

			jobEdit.setEnabled(false);
			runBackup.setEnabled(false);
			runRestore.setEnabled(false);
			runPurge.setEnabled(false);
			runListFiles.setEnabled(false);
			runStop.setEnabled(true);
		} 
	}

}
