/**
 * TERMS AND CONDITIONS:
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.backup.zipsnap.ng.gui;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import javax.swing.JTabbedPane;
import javax.swing.JLabel;
import javax.swing.JCheckBox;
import javax.swing.JTextField;

import com.backup.zipsnap.ng.Main;
import com.backup.zipsnap.ng.gui.util.DialogFrame;

import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.ImageIcon;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.awt.Toolkit;

public class SettingsFrame extends JFrame {

	private static final long serialVersionUID = 6613112575241723983L;
	private GUI gui;
	private JFrame thisFrame;
	private JPanel contentPane;
	private JTextField summaryMaxFiles;
	private JTextField timeTolerance;
	private JTextField progressBarStep;
	private JTextField bufferSize;
	private JTextField sliceSize;
	private JCheckBox deepFileAnalysis;
	private JCheckBox confirmExit;
	private JCheckBox restoreOpenedTabs;
	private JCheckBox guiDebug;
	@SuppressWarnings("rawtypes")
	private JComboBox debug;
	private JCheckBox memoryStats;
	private JCheckBox pauseOnWarning;
	@SuppressWarnings("rawtypes")
	private JComboBox overwriteExisting;
	@SuppressWarnings("rawtypes")
	private JComboBox deleteExisting;
	private JCheckBox purgeOldCatalogFiles;
	private JCheckBox purgeAfterBackup;
	private JCheckBox logDebugAlways;
	private JCheckBox logFlushAlways;
	private JCheckBox verifyAfterRestore;
	private JCheckBox listFilesOnSummary;
	private JCheckBox deleteAfterRestore;
	private JCheckBox hideProgressDuringBatch;
	private JCheckBox catalogDeepCheck;
	private JCheckBox printWarningsSummary;
	private JCheckBox infoAfterBackup;
	private JCheckBox outputActionWhenBackupEmpty;
	private JCheckBox removeInvalidSlicesAfterBackup;
	private JPanel ButtonPanel;
	private JTextField fileTreeMaxItems;
	private JTextField textAreaMaxItems;
	private JTextField maxSlicesPerVolume;
	private JTextField maxVolumeSize;
	private JLabel label;
	private JLabel lblMaximumSizeOf;

	/*
	 * Create the frame.
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public SettingsFrame(GUI gui) {
		setResizable(false);
		setIconImage(Toolkit.getDefaultToolkit().getImage(SettingsFrame.class.getResource("/com/sun/java/swing/plaf/windows/icons/File.gif")));
		this.thisFrame = this;
		this.gui = gui;
		setBounds(0, 0, 769, 602);
		setTitle("Settings");
		contentPane = new JPanel();
		contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		setContentPane(contentPane);
		contentPane.setLayout(null);
		
		JTabbedPane tabbedPane = new JTabbedPane(JTabbedPane.TOP);
		tabbedPane.setBounds(5, 5, 741, 516);
		contentPane.add(tabbedPane);
		
		JPanel guiTab = new JPanel();
		tabbedPane.addTab("GUI Settings", null, guiTab, null);
		guiTab.setLayout(null);
		
		confirmExit = new JCheckBox("Ask confirmation before exiting");
		confirmExit.setBounds(8, 9, 233, 25);
		guiTab.add(confirmExit);
		
		restoreOpenedTabs = new JCheckBox("Restore previously opened tabs");
		restoreOpenedTabs.setBounds(8, 39, 347, 25);
		guiTab.add(restoreOpenedTabs);
		
		guiDebug = new JCheckBox("Enable GUI debug");
		guiDebug.setBounds(8, 69, 200, 25);
		guiTab.add(guiDebug);
		
		fileTreeMaxItems = new JTextField();
		fileTreeMaxItems.setBounds(12, 103, 72, 22);
		guiTab.add(fileTreeMaxItems);
		fileTreeMaxItems.setColumns(10);
		
		textAreaMaxItems = new JTextField();
		textAreaMaxItems.setColumns(10);
		textAreaMaxItems.setBounds(12, 139, 72, 22);
		guiTab.add(textAreaMaxItems);
		
		JLabel lblNewLabel_6 = new JLabel("Maximum number of items to show into a File Tree");
		lblNewLabel_6.setBounds(96, 106, 502, 16);
		guiTab.add(lblNewLabel_6);
		
		JLabel lblNewLabel_7 = new JLabel("Maximum number of items to show into a Text Area");
		lblNewLabel_7.setBounds(96, 142, 415, 16);
		guiTab.add(lblNewLabel_7);
		
		JPanel cliTab = new JPanel();
		tabbedPane.addTab("CLI Settings", null, cliTab, null);
		cliTab.setLayout(null);
		
		debug = new JComboBox();
		debug.addItem("no");
		debug.addItem("yes");
		debug.addItem("verbose");
		debug.addItem("all");
		debug.setBounds(12, 13, 98, 22);
		cliTab.add(debug);
		
		JLabel lblEnableCliDebug = new JLabel("Enable CLI Debug");
		lblEnableCliDebug.setBounds(125, 16, 176, 16);
		cliTab.add(lblEnableCliDebug);
		
		memoryStats = new JCheckBox("Calculate and print memory usage statistics. Requires Classmexer java agent to be loaded");
		memoryStats.setBounds(12, 44, 685, 25);
		cliTab.add(memoryStats);
		
		timeTolerance = new JTextField();
		timeTolerance.setBounds(12, 78, 58, 22);
		cliTab.add(timeTolerance);
		timeTolerance.setColumns(10);
		
		JLabel lblNewLabel = new JLabel("Time between the current time and the file timestamp to determine if has been changed");
		lblNewLabel.setBounds(109, 81, 610, 16);
		cliTab.add(lblNewLabel);
		
		pauseOnWarning = new JCheckBox("Pause on each warning");
		pauseOnWarning.setBounds(12, 109, 455, 25);
		cliTab.add(pauseOnWarning);
		
		overwriteExisting = new JComboBox();
		overwriteExisting.addItem("no");
		overwriteExisting.addItem("prompt");
		overwriteExisting.addItem("yes");
		overwriteExisting.setBounds(12, 143, 98, 22);
		cliTab.add(overwriteExisting);
		
		deleteExisting = new JComboBox();
		deleteExisting.addItem("no");
		deleteExisting.addItem("prompt");
		deleteExisting.addItem("yes");
		deleteExisting.setBounds(12, 178, 98, 22);
		cliTab.add(deleteExisting);
		
		JLabel lblNewLabel_1 = new JLabel("Overwrite existing files/directories when restoring a snapshot");
		lblNewLabel_1.setBounds(125, 146, 594, 16);
		cliTab.add(lblNewLabel_1);
		
		JLabel lblNewLabel_2 = new JLabel("Delete unmatched existing files/directories when restoring a snapshot");
		lblNewLabel_2.setBounds(125, 181, 594, 16);
		cliTab.add(lblNewLabel_2);
		
		progressBarStep = new JTextField();
		progressBarStep.setBounds(12, 213, 58, 22);
		cliTab.add(progressBarStep);
		progressBarStep.setColumns(10);
		
		JLabel lblNewLabel_3 = new JLabel("Show the progress bar every %");
		lblNewLabel_3.setBounds(109, 216, 278, 16);
		cliTab.add(lblNewLabel_3);
		
		bufferSize = new JTextField();
		bufferSize.setBounds(12, 245, 40, 22);
		cliTab.add(bufferSize);
		bufferSize.setColumns(10);
		
		sliceSize = new JTextField();
		sliceSize.setBounds(12, 280, 40, 22);
		cliTab.add(sliceSize);
		sliceSize.setColumns(10);
		
		JLabel lblKb = new JLabel("Kb");
		lblKb.setBounds(53, 248, 21, 16);
		cliTab.add(lblKb);
		
		JLabel lblMb = new JLabel("Mb");
		lblMb.setBounds(53, 283, 56, 16);
		cliTab.add(lblMb);
		
		JLabel lblNewLabel_4 = new JLabel("Size of the buffer for read/write operations");
		lblNewLabel_4.setBounds(109, 248, 380, 16);
		cliTab.add(lblNewLabel_4);
		
		JLabel lblNewLabel_5 = new JLabel("Size of a data slice (WARNING: changing this value would require a new full backup)");
		lblNewLabel_5.setBounds(109, 283, 588, 16);
		cliTab.add(lblNewLabel_5);
		
		JLabel lblS = new JLabel("s");
		lblS.setBounds(73, 81, 56, 16);
		cliTab.add(lblS);
		
		maxSlicesPerVolume = new JTextField();
		maxSlicesPerVolume.setBounds(12, 350, 73, 22);
		cliTab.add(maxSlicesPerVolume);
		maxSlicesPerVolume.setColumns(10);
		
		JLabel lblMaximumNumberOf = new JLabel("Maximum number of data slices for each backup volume. \"0\" for unlimited");
		lblMaximumNumberOf.setBounds(109, 353, 631, 16);
		cliTab.add(lblMaximumNumberOf);
		
		maxVolumeSize = new JTextField();
		maxVolumeSize.setBounds(12, 315, 65, 22);
		cliTab.add(maxVolumeSize);
		maxVolumeSize.setColumns(10);
		
		label = new JLabel("Mb");
		label.setBounds(80, 318, 56, 16);
		cliTab.add(label);
		
		lblMaximumSizeOf = new JLabel("Maximum size of each backup volume. \"0\" for unlimited");
		lblMaximumSizeOf.setBounds(109, 315, 631, 16);
		cliTab.add(lblMaximumSizeOf);
		
		JPanel advancedTab = new JPanel();
		tabbedPane.addTab("Advanced Features", null, advancedTab, null);
		advancedTab.setLayout(null);
		
		purgeOldCatalogFiles = new JCheckBox("Old catalog files will be deleted upon an archive purge");
		purgeOldCatalogFiles.setBounds(8, 9, 517, 25);
		advancedTab.add(purgeOldCatalogFiles);
		
		purgeAfterBackup = new JCheckBox("The archive will be purged once the backup is finished");
		purgeAfterBackup.setBounds(8, 39, 488, 25);
		advancedTab.add(purgeAfterBackup);
		
		logDebugAlways = new JCheckBox("Log file contains always debug information");
		logDebugAlways.setBounds(8, 69, 517, 25);
		advancedTab.add(logDebugAlways);
		
		logFlushAlways = new JCheckBox("Do not buffer log entries but flush to file immediately");
		logFlushAlways.setBounds(8, 99, 621, 25);
		advancedTab.add(logFlushAlways);
		
		verifyAfterRestore = new JCheckBox("Verify the restored files after the extraction");
		verifyAfterRestore.setBounds(8, 129, 598, 25);
		advancedTab.add(verifyAfterRestore);
		
		listFilesOnSummary = new JCheckBox("Show the full list of files during backup/restore in the summary");
		listFilesOnSummary.setBounds(8, 159, 621, 25);
		advancedTab.add(listFilesOnSummary);
		
		summaryMaxFiles = new JTextField();
		summaryMaxFiles.setBounds(111, 193, 56, 22);
		advancedTab.add(summaryMaxFiles);
		summaryMaxFiles.setColumns(10);
		
		JLabel lblAximumNumberOf = new JLabel("Maximum number of files to show in the summary. \"0\" for all");
		lblAximumNumberOf.setBounds(179, 193, 460, 16);
		advancedTab.add(lblAximumNumberOf);
		
		deleteAfterRestore = new JCheckBox("After restoring, delete from the destination directory the files not contained in the snapshots");
		deleteAfterRestore.setBounds(8, 224, 675, 25);
		advancedTab.add(deleteAfterRestore);
		
		hideProgressDuringBatch = new JCheckBox("Hide the progress bar when running as a batch job");
		hideProgressDuringBatch.setBounds(8, 254, 647, 25);
		advancedTab.add(hideProgressDuringBatch);
		
		catalogDeepCheck = new JCheckBox("Perform an in-depth integrity check by verifying that each data slice is in the ZIP archive");
		catalogDeepCheck.setBounds(8, 284, 633, 25);
		advancedTab.add(catalogDeepCheck);
		
		printWarningsSummary = new JCheckBox("Print a summary of all the warnings at the end of the job");
		printWarningsSummary.setBounds(8, 314, 422, 25);
		advancedTab.add(printWarningsSummary);
		
		infoAfterBackup = new JCheckBox("Run an archive info when the backup process is finished");
		infoAfterBackup.setBounds(8, 344, 439, 25);
		advancedTab.add(infoAfterBackup);
		
		outputActionWhenBackupEmpty = new JCheckBox("Run the configured custom command even if the snapshot does not contain any file");
		outputActionWhenBackupEmpty.setBounds(8, 374, 633, 25);
		advancedTab.add(outputActionWhenBackupEmpty);
		
		removeInvalidSlicesAfterBackup = new JCheckBox("Remove from the ZIP archives slices marked as invalid");
		removeInvalidSlicesAfterBackup.setBounds(8, 404, 675, 25);
		advancedTab.add(removeInvalidSlicesAfterBackup);
		
		deepFileAnalysis = new JCheckBox("Use the checksum to verify if a file has been changed");
		deepFileAnalysis.setBounds(8, 434, 675, 25);
		advancedTab.add(deepFileAnalysis);
		
		ButtonPanel = new JPanel();
		ButtonPanel.setBounds(5, 513, 741, 44);
		contentPane.add(ButtonPanel);
		ButtonPanel.setLayout(null);
		
		JButton btnCancel = new JButton("Cancel");
		btnCancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				thisFrame.setVisible(false);
			}
		});
		btnCancel.setBounds(532, 13, 97, 25);
		ButtonPanel.add(btnCancel);
		
		JButton btnSave = new JButton("Save");
		btnSave.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				saveConfig();
			}
		});
		btnSave.setIcon(new ImageIcon(SettingsFrame.class.getResource("/com/sun/java/swing/plaf/windows/icons/FloppyDrive.gif")));
		btnSave.setBounds(644, 13, 97, 25);
		ButtonPanel.add(btnSave);
		
		JButton btnRestoreDefaults = new JButton("Restore defaults");
		btnRestoreDefaults.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				loadDefaults();
			}
		});
		btnRestoreDefaults.setBounds(0, 13, 163, 25);
		ButtonPanel.add(btnRestoreDefaults);
	}
	
	/*
	 * load the settings into the widgets
	 */
	public void load() {
		if (Main.settings.properties.get("confirmExit").equals("true")) confirmExit.setSelected(true);
		if (Main.settings.properties.get("restoreOpenedTabs").equals("true")) restoreOpenedTabs.setSelected(true);
		if (Main.settings.properties.get("guiDebug").equals("true")) guiDebug.setSelected(true);
		fileTreeMaxItems.setText(new Integer((String) Main.settings.properties.get("fileTreeMaxItems")).intValue()+"");
		textAreaMaxItems.setText(new Integer((String) Main.settings.properties.get("textAreaMaxItems")).intValue()+"");

		int debugIndex = 0;
		if (Main.settings.properties.get("debug").equals("no")) debugIndex = 0;
		if (Main.settings.properties.get("debug").equals("yes")) debugIndex = 1;
		if (Main.settings.properties.get("debug").equals("verbose")) debugIndex = 2;
		if (Main.settings.properties.get("debug").equals("all")) debugIndex = 3;
		debug.setSelectedIndex(debugIndex);
		if (Main.settings.properties.get("memoryStats").equals("true")) memoryStats.setSelected(true);
		timeTolerance.setText(new Integer((String)Main.settings.properties.get("timeTolerance")).intValue()+"");
		if (Main.settings.properties.get("pauseOnWarning").equals("true")) pauseOnWarning.setSelected(true);
		overwriteExisting.setSelectedItem(Main.settings.properties.get("overwriteExisting"));
		deleteExisting.setSelectedItem(Main.settings.properties.get("deleteExisting"));
		progressBarStep.setText(new Integer((String) Main.settings.properties.get("progressBarStep")).intValue()+"");
		bufferSize.setText(new Integer((String) Main.settings.properties.get("bufferSize")).intValue()+"");
		sliceSize.setText(new Integer((String) Main.settings.properties.get("sliceSize")).intValue()+"");
		maxVolumeSize.setText(new Integer((String) Main.settings.properties.get("maxVolumeSize")).intValue()+"");
		maxSlicesPerVolume.setText(new Integer((String) Main.settings.properties.get("maxSlicesPerVolume")).intValue()+"");
		
		if (Main.settings.properties.get("purgeOldCatalogFiles").equals("true")) purgeOldCatalogFiles.setSelected(true);
		if (Main.settings.properties.get("purgeAfterBackup").equals("true")) purgeAfterBackup.setSelected(true);
		if (Main.settings.properties.get("logDebugAlways").equals("true")) logDebugAlways.setSelected(true);
		if (Main.settings.properties.get("logFlushAlways").equals("true")) logFlushAlways.setSelected(true);
		if (Main.settings.properties.get("verifyAfterRestore").equals("true")) verifyAfterRestore.setSelected(true);
		if (Main.settings.properties.get("listFilesOnSummary").equals("true")) listFilesOnSummary.setSelected(true);
		summaryMaxFiles.setText(new Integer((String) Main.settings.properties.get("summaryMaxFiles")).intValue()+"");		
		if (Main.settings.properties.get("deleteAfterRestore").equals("true")) deleteAfterRestore.setSelected(true);
		if (Main.settings.properties.get("hideProgressDuringBatch").equals("true")) hideProgressDuringBatch.setSelected(true);
		if (Main.settings.properties.get("catalogDeepCheck").equals("true")) catalogDeepCheck.setSelected(true);
		if (Main.settings.properties.get("printWarningsSummary").equals("true")) printWarningsSummary.setSelected(true);
		if (Main.settings.properties.get("infoAfterBackup").equals("true")) infoAfterBackup.setSelected(true);
		if (Main.settings.properties.get("outputActionWhenBackupEmpty").equals("true")) outputActionWhenBackupEmpty.setSelected(true);
		if (Main.settings.properties.get("removeInvalidSlicesAfterBackup").equals("true")) removeInvalidSlicesAfterBackup.setSelected(true);
		if (Main.settings.properties.get("deepFileAnalysis").equals("true")) deepFileAnalysis.setSelected(true);
		gui.windowsManager.show(thisFrame);
	}
	
	/*
	 * load the default settings
	 */
	private void loadDefaults() {
		confirmExit.setSelected(true);
		restoreOpenedTabs.setSelected(true);
		guiDebug.setSelected(false);
		fileTreeMaxItems.setText(10000+"");
		textAreaMaxItems.setText(10000+"");

		debug.setSelectedIndex(0);
		memoryStats.setSelected(false);
		timeTolerance.setText(10+"");
		pauseOnWarning.setSelected(false);
		overwriteExisting.setSelectedItem("prompt");
		deleteExisting.setSelectedItem("prompt");
		progressBarStep.setText(10+"");
		bufferSize.setText(64+"");
		sliceSize.setText(1+"");
		maxVolumeSize.setText(1024+"");
		maxSlicesPerVolume.setText(25000+"");
		
		purgeOldCatalogFiles.setSelected(false);
		purgeAfterBackup.setSelected(true);
		logDebugAlways.setSelected(true);
		logFlushAlways.setSelected(true);
		verifyAfterRestore.setSelected(true);
		listFilesOnSummary.setSelected(true);
		summaryMaxFiles.setText(0+"");
		deleteAfterRestore.setSelected(true);
		hideProgressDuringBatch.setSelected(false);
		catalogDeepCheck.setSelected(true);
		printWarningsSummary.setSelected(true);
		infoAfterBackup.setSelected(true);
		outputActionWhenBackupEmpty.setSelected(true);
		removeInvalidSlicesAfterBackup.setSelected(true);
		deepFileAnalysis.setSelected(false);
	}
	
	/*
	 * check the settings and save 	
	 */
	private void saveConfig() {
		ArrayList<String> errors = new ArrayList<String>();
		if (! timeTolerance.getText().matches("[0-9]+")) errors.add("Time tolerance must be a number");
		if (! progressBarStep.getText().matches("[0-9]+")) errors.add("The progress bar step must be a number");
		if (! bufferSize.getText().matches("[0-9]+")) errors.add("The buffer size must be a number");
		if (! sliceSize.getText().matches("[0-9]+")) errors.add("The slice size must be a number");
		if (! maxVolumeSize.getText().matches("[0-9]+")) errors.add("The maximum volume size must be a number");
		if (! maxSlicesPerVolume.getText().matches("[0-9]+")) errors.add("The maximum number of data slices for each volume must be a number");
		
		if (! summaryMaxFiles.getText().matches("[0-9]+")) errors.add("The maximum number of files in the summary must be a number");
		if (! fileTreeMaxItems.getText().matches("[0-9]+")) errors.add("The maximum number of items for a file tree must be a number");
		if (! textAreaMaxItems.getText().matches("[0-9]+")) errors.add("The maximum number of items for a text area must be a number");
		if (errors.size() == 0) {
			/* save the config */
			applyConfig();
			thisFrame.setVisible(false);
		}
		else {
			/* show an error message */
			String errorString = "";
			for (String s : errors) errorString = errorString + s+"\n";
			gui.windowsManager.show(new DialogFrame(DialogFrame.type.ERROR, errorString));
		}
	}
	
	/*
	 * save the settings
	 */
	private void applyConfig() {
		/* save GUI settings */
		Main.settings.confirmExit = confirmExit.isSelected() ? true : false;
		Main.settings.restoreOpenedTabs = restoreOpenedTabs.isSelected() ? true : false;
		Main.settings.guiDebug = guiDebug.isSelected() ? true : false;
		Main.settings.fileTreeMaxItems = new Integer(fileTreeMaxItems.getText()).intValue();
		Main.settings.textAreaMaxItems = new Integer(textAreaMaxItems.getText()).intValue();
		
		/* save CLI settings */
		Main.settings.debug = debug.getSelectedIndex();
		Main.settings.memoryStats = memoryStats.isSelected() ? true : false;
		Main.settings.timeTolerance = new Integer(timeTolerance.getText()).intValue();
		Main.settings.pauseOnWarning = pauseOnWarning.isSelected() ? true : false;
		Main.settings.overwriteExisting = (String)overwriteExisting.getSelectedItem();
		Main.settings.deleteExisting = (String)deleteExisting.getSelectedItem();
		Main.settings.progressBarStep = new Integer(progressBarStep.getText()).intValue();
		Main.settings.bufferSize = new Integer(bufferSize.getText()).intValue()*Main.settings.Kb;
		Main.settings.sliceSize = new Integer(sliceSize.getText()).intValue()*Main.settings.Mb;
		Main.settings.maxVolumeSize = new Integer(maxVolumeSize.getText()).intValue()*Main.settings.Mb;
		Main.settings.maxSlicesPerVolume = new Integer(maxSlicesPerVolume.getText()).intValue();

		/* save advanced settings */		
		Main.settings.purgeOldCatalogFiles = purgeOldCatalogFiles.isSelected() ? true : false;
		Main.settings.purgeAfterBackup = purgeAfterBackup.isSelected() ? true : false;
		Main.settings.logDebugAlways = logDebugAlways.isSelected() ? true : false;
		Main.settings.logFlushAlways = logFlushAlways.isSelected() ? true : false;
		Main.settings.verifyAfterRestore = verifyAfterRestore.isSelected() ? true : false;
		Main.settings.listFilesOnSummary = listFilesOnSummary.isSelected() ? true : false;
		Main.settings.summaryMaxFiles = new Integer(summaryMaxFiles.getText()).intValue();
		Main.settings.deleteAfterRestore = deleteAfterRestore.isSelected() ? true : false;
		Main.settings.hideProgressDuringBatch = hideProgressDuringBatch.isSelected() ? true : false;
		Main.settings.catalogDeepCheck = catalogDeepCheck.isSelected() ? true : false;
		Main.settings.printWarningsSummary = printWarningsSummary.isSelected() ? true : false;
		Main.settings.infoAfterBackup = infoAfterBackup.isSelected() ? true : false;
		Main.settings.outputActionWhenBackupEmpty = outputActionWhenBackupEmpty.isSelected() ? true : false;
		Main.settings.removeInvalidSlicesAfterBackup = removeInvalidSlicesAfterBackup.isSelected() ? true : false;
		Main.settings.deepFileAnalysis = deepFileAnalysis.isSelected() ? true : false;
		
		/* save the properties to disk */
		Main.settings.save();
	}
}
