/**
 * TERMS AND CONDITIONS:
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.backup.zipsnap.ng.gui;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.border.BevelBorder;

import com.backup.zipsnap.ng.Main;
import com.backup.zipsnap.ng.gui.util.DialogFrame;
import com.backup.zipsnap.ng.gui.util.TextFrame;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

public class StatusBar extends JPanel {

	private static final long serialVersionUID = -6684865718649370198L;
	private JLabel warningLabel;
	private JLabel errorLabel;
	private JLabel infoLabel;
	private TextFrame warningFrame;
	private TextFrame errorFrame;
	private TextFrame infoFrame;
	private GUI gui;
	private String defaultLabel = Main.programTitle;
	private JLabel label;
	public static enum status { RUNNING,READY; }


	/*
	 * Create the status bar
	 */
	public StatusBar(final GUI gui) {
		this.gui = gui;
		setBorder(new BevelBorder(BevelBorder.LOWERED, null, null, null, null));
		setBounds(0, 659, 1006, 31);
		setLayout(null);

		/* init the label panel */
		JPanel labelPanel = new JPanel();
		labelPanel.setBorder(new BevelBorder(BevelBorder.LOWERED, null, null, null, null));
		labelPanel.setBounds(0, 0, 761, 31);
		add(labelPanel);
		labelPanel.setLayout(null);
		label = new JLabel(defaultLabel);
		label.setBounds(12, 0, 612, 29);
		labelPanel.add(label);
		
		/* init the messages frames */
		warningFrame = new TextFrame("Warning Messages");
		errorFrame = new TextFrame("Error Messages");
		infoFrame = new TextFrame("Info Messages");
		
		/* init the message panel */
		JPanel messagesPanel = new JPanel();
		messagesPanel.setBorder(new BevelBorder(BevelBorder.LOWERED, null, null, null, null));
		messagesPanel.setBounds(760, 0, 246, 31);
		messagesPanel.setLayout(null);
		add(messagesPanel);
		
		/* create the message icons */
		warningLabel = new JLabel("0");
		warningLabel.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent arg0) {
				gui.windowsManager.show(warningFrame);
			}
		});		
		ImageIcon warningIcon = new ImageIcon(StatusBar.class.getResource("/javax/swing/plaf/metal/icons/Warn.gif"));
		warningLabel.setIcon(new ImageIcon(warningIcon.getImage().getScaledInstance(16, 16, java.awt.Image.SCALE_SMOOTH)));
		warningLabel.setHorizontalAlignment(SwingConstants.LEFT);
		warningLabel.setBounds(70, 0, 54, 29);
		messagesPanel.add(warningLabel);

		errorLabel = new JLabel("0");
		errorLabel.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				gui.windowsManager.show(errorFrame);
			}
		});
		ImageIcon errorIcon = new ImageIcon(StatusBar.class.getResource("/javax/swing/plaf/metal/icons/Error.gif"));
		errorLabel.setIcon(new ImageIcon(errorIcon.getImage().getScaledInstance(16, 16, java.awt.Image.SCALE_SMOOTH)));
		errorLabel.setHorizontalAlignment(SwingConstants.LEFT);
		errorLabel.setBounds(12, 0, 54, 29);
		messagesPanel.add(errorLabel);
		
		infoLabel = new JLabel("0");
		infoLabel.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent arg0) {
				gui.windowsManager.show(infoFrame);
			}
		});
		ImageIcon infoIcon = new ImageIcon(StatusBar.class.getResource("/javax/swing/plaf/metal/icons/Inform.gif"));
		infoLabel.setIcon(new ImageIcon(infoIcon.getImage().getScaledInstance(16, 16, java.awt.Image.SCALE_SMOOTH)));
		infoLabel.setHorizontalAlignment(SwingConstants.LEFT);
		infoLabel.setBounds(129, 0, 89, 29);
		messagesPanel.add(infoLabel);
		
		setStatus(status.READY);
	}
	
	/*
	 * reset the status bar
	 */
	public void reset() {
		warningLabel.setText("0");
		errorLabel.setText("0");
		infoLabel.setText("0");
		warningFrame.reset();
		errorFrame.reset();
		infoFrame.reset();
	}
	
	/*
	 * set the status label
	 */
	public void setLabel(String msg) {
		label.setText(msg);
	}
	
	/*
	 * add a message
	 */
	public void setMessage(String what, String msg) {
		TextFrame frame = null;
		JLabel label = null;
		if (what == "warning") {
			frame = warningFrame;
			label = warningLabel;
		} 
		else if (what == "error") {
			frame = errorFrame;
			label = errorLabel;
			gui.windowsManager.show(new DialogFrame(DialogFrame.type.ERROR,msg));
		}
		else if (what == "info") {
			frame = infoFrame;
			label = infoLabel;
		}
		/* increase the counter */
		int count = 0;
		try {
			count = Integer.parseInt(label.getText());
		} catch (Exception e) {
		}
		count++;
		label.setText(new String(count+""));
		if (msg.length() > 2 && ! msg.contains("%") && ! msg.contains("[") && ! msg.contains("]")) label.setToolTipText(msg);
		/* add the text */
		frame.append(msg);
	}
	
	/*
	 * return the number of messages of a given label
	 */
	public int getMessageCount(String what) {
		JLabel label = null;
		if (what == "warning") label = warningLabel;
		if (what == "error") label = errorLabel;
		if (what == "info")  label = infoLabel;
		int count = 0;
		try {
			count = Integer.parseInt(label.getText());
		} catch (Exception e) {
		}
		return count;
	}
	
	/*
	 * receive status updates
	 */
	public void setStatus(status what) {
		if (what == status.READY) {
			label.setText(defaultLabel);
		}
		else if (what == status.RUNNING) {
			label.setText("Running...");
		} 
	}
}
