/**
 * TERMS AND CONDITIONS:
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.backup.zipsnap.ng.gui;

import java.io.File;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JPanel;

import com.backup.zipsnap.ng.gui.util.LoadingPanel;
import com.backup.zipsnap.ng.job.Config;

public class Tab extends JPanel implements Observer {

	private static final long serialVersionUID = 8015442143585888415L;
	private LoadingPanel loadingPanel;
	private GUI gui;
	public File configFile;
	public JobPanel jobPanel;
	public StatusBar statusBar;
	public Config config;
	public JobWrapper jobEngine;
	public enum action {LOAD,CREATE,RESTORE};
	public JobConfigFrame configFrame;
	
	/*
	 * Create the panel.
	 */
	
	public Tab(GUI gui, File configFile) {
		this.configFile = configFile;
		this.gui = gui;
		GUI.print("New tab: "+getName()+" from config file "+configFile.getPath());
		
	    /* set the layout */
	    setLayout(null);
	    setBounds(0, 0, 1014, 710);
	    
	    /* create and add the job panel */
	    jobPanel = new JobPanel(gui,this,configFile);
		jobPanel.setVisible(false);
		add(jobPanel);

		/* create and add the loading panel */
	    loadingPanel = new LoadingPanel();
	    loadingPanel.setBounds(0, 0, 1014, 650);
	    loadingPanel.setVisible(false);
	    add(loadingPanel);
		
		/* draw the status bar */
		statusBar = new StatusBar(gui);
		add(statusBar);
		
		/* instantiate a jobEngine for running commands */
	    jobEngine = new JobWrapper(gui,this);
		jobEngine.addObserver(this);
		
		/* instantiate a configFrame for the job's settings */
		configFrame = new JobConfigFrame(this.gui,this,getName());
	}
	
	/*
	 * return the name of this job
	 */
	public String getName() {
		String name = configFile.getName();
	    if (name.indexOf(".") > 0) name = name.substring(0, name.lastIndexOf("."));
	    return name;
	}
	
	/*
	 * load the job
	 */
	public void actionLoadJob() {
		statusLoading();
		String cmd[] = {configFile.getAbsolutePath(),"info"};
		runJob(cmd);
	}
	
	/*
	 * update the configuration of the current tab
	 */
	public void actionUpdateJob() {
		statusLoading();
		String cmd[] = {configFile.getAbsolutePath(),"updateConfig"};
		runJob(cmd);
		actionLoadJob();
	}
	
	/*
	 * create a new empty config file
	 */
	public void actionCreateJob() {
		statusLoading();
		String cmd[] = {configFile.getAbsolutePath(),"config"};
		runJob(cmd);
		actionEditJobConfig(true);
	}
	
	/*
	 * show the configuration panel
	 */
	public void actionEditJobConfig(boolean createJob) {
		if (!createJob) configFrame.loadConfig();
		gui.windowsManager.show(configFrame);
	}

	/*
	 * job is loading
	 */
	private void statusLoading() {
		/* show the loading panel */
		jobPanel.reset();
		jobPanel.setVisible(false);
		loadingPanel.setVisible(true);
	}

	/*
	 * job is loaded
	 */
	private void statusReady() {
		loadingPanel.setVisible(false);
		jobPanel.setVisible(true);
	}
	
	/*
	 * run a job
	 */
	public void runJob(String[] args) {
		statusBar.reset();
		jobEngine.run(args);
	}
	
	
	/*
	 * receive updates from other objects
	 */
	public void update(Observable classObj, Object arg1) {
		/* the job has just started running */ 
		if (classObj == jobEngine && jobEngine.isRunning) {
			/* show a loading panel. Do not show if running a taskPanel */ 
			if (! jobEngine.command.equals("backup") && ! jobEngine.command.equals("restore") && ! jobEngine.command.equals("purge") && ! jobEngine.command.equals("infoSnapshot")) statusLoading();
		}
		/* the job has just stopped running */
		if (classObj == jobEngine && ! jobEngine.isRunning) {
			statusReady();
		}
	}
}
