/**
 * TERMS AND CONDITIONS:
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.backup.zipsnap.ng.gui.util;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.UIManager;


public class DialogFrame extends JFrame {
	private static final long serialVersionUID = 6288955679578311353L;
	private final JPanel contentPanel = new JPanel();
	private type thisType;
	public static enum type { INFO,WARNING,ERROR,YES_NO,YES_NO_ALWAYS_NEVER,OK; }
	public static enum action { OK,YES,NO,ALWAYS,NEVER; }


	/*
	 * Create the dialog panel
	 */
	public DialogFrame(type type, String message) {
		setResizable(false);
		this.thisType = type;
		setBounds(0, 0, 438, 199);
		/* set the layout */
		setIconImage(Toolkit.getDefaultToolkit().getImage(WindowsManager.class.getResource(getDialogIcon())));
		setTitle(getDialogTitle());
		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		getContentPane().setLayout(new BorderLayout());
		contentPanel.setBorder(null);
		getContentPane().add(contentPanel, BorderLayout.CENTER);
		contentPanel.setLayout(null);
		{
			JScrollPane scrollPane = new JScrollPane();
			scrollPane.setBounds(10, 13, 398, 93);
			/* add the text area */
			contentPanel.add(scrollPane);
			{
				JTextArea textArea = new JTextArea(message);
				textArea.setBackground(UIManager.getColor("Button.background"));
				textArea.setWrapStyleWord(true);
				textArea.setLineWrap(true);
				textArea.setEditable(false);
				scrollPane.setViewportView(textArea);
			}
		}
		{
			/* add the buttons */
			JPanel buttonPane = new JPanel();
			buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
			getContentPane().add(buttonPane, BorderLayout.SOUTH);
			getButtons(buttonPane);
		}
		setVisible(true);
	}
	
	/*
	 * return the icon based on the dialog type
	 */
	private String getDialogIcon() {
		if (thisType == type.INFO) return "/javax/swing/plaf/metal/icons/Inform.gif";
		if (thisType == type.WARNING) return "/javax/swing/plaf/metal/icons/Warn.gif";
		if (thisType == type.ERROR) return "/javax/swing/plaf/metal/icons/Error.gif";
		if (thisType == type.YES_NO) return "/javax/swing/plaf/metal/icons/Question.gif";
		if (thisType == type.YES_NO_ALWAYS_NEVER) return "/javax/swing/plaf/metal/icons/Question.gif";
		if (thisType == type.OK) return "/javax/swing/plaf/metal/icons/Inform.gif";
		return "/javax/swing/plaf/metal/icons/Inform.gif";
	}

	/*
	 * return the title based on the dialog type
	 */
	private String getDialogTitle() {
		if (thisType == type.INFO) return "Info";
		if (thisType == type.WARNING) return "Warning";
		if (thisType == type.ERROR) return "Error";
		if (thisType == type.YES_NO) return "Confirm";
		if (thisType == type.YES_NO_ALWAYS_NEVER) return "Confirm";
		if (thisType == type.OK) return "Info";
		return "/javax/swing/plaf/metal/icons/Inform.gif";
	}

	/*
	 * add the  buttons based on the dialog type
	 */
	private void getButtons(JPanel buttonPane) {
		if (thisType == type.INFO || thisType == type.WARNING || thisType == type.ERROR) {
			JButton okButton = new JButton("OK");
			buttonPane.add(okButton);
			getRootPane().setDefaultButton(okButton);
			okButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
					firePropertyChange("action",null, action.OK);
					dispose();
				}
			});
		}
		else if (thisType == type.YES_NO) {
			JButton yesButton = new JButton("Yes");
			JButton noButton = new JButton("No");
			buttonPane.add(yesButton);
			buttonPane.add(noButton);
			getRootPane().setDefaultButton(yesButton);
			yesButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
					firePropertyChange("action",null, action.YES);
					dispose();
				}
			});
			noButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
					firePropertyChange("action",null, action.NO);
					dispose();
				}
			});			
		}
		else if (thisType == type.OK) {
			JButton okButton = new JButton("Ok");
			buttonPane.add(okButton);
			getRootPane().setDefaultButton(okButton);
			okButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
					firePropertyChange("action",null, action.OK);
					dispose();
				}
			});
		}
		else if (thisType == type.YES_NO_ALWAYS_NEVER) {
			JButton yesButton = new JButton("Yes");
			JButton noButton = new JButton("No");
			JButton alwaysButton = new JButton("Always");
			JButton neverButton = new JButton("Never");

			buttonPane.add(yesButton);
			buttonPane.add(alwaysButton);
			buttonPane.add(noButton);
			buttonPane.add(neverButton);
			getRootPane().setDefaultButton(yesButton);
			yesButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
					firePropertyChange("action",null, action.YES);
					dispose();
				}
			});
			noButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
					firePropertyChange("action",null, action.NO);
					dispose();
				}
			});			
			alwaysButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
					firePropertyChange("action",null, action.ALWAYS);
					dispose();
				}
			});
			neverButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
					firePropertyChange("action",null, action.NEVER);
					dispose();
				}
			});			

		}

	}




}


