/**
 * TERMS AND CONDITIONS:
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.backup.zipsnap.ng.job;

import java.util.Arrays;
import java.util.List;
import java.util.ArrayList;
import java.sql.Timestamp;
import java.util.Date;
import java.util.HashMap;

import com.backup.zipsnap.ng.Main;
import com.backup.zipsnap.ng.gui.JobWrapper;
import com.backup.zipsnap.ng.job.unit.FileUnit;
import com.backup.zipsnap.ng.job.util.FileIO;
import com.backup.zipsnap.ng.job.util.UserIO;
/**
 * Provide info about an archive 
 */
public class Info
{
  
  /**
   * Display information about the ZipSnap archive.
   */
  public static void archiveSummary(boolean directCall)
  {
    if (directCall) {
      UserIO.print(UserIO.out.INFO,"Requested action: BACKUP INFO");
      
      /****************************************
        * (1) READ CATALOG FOR THE ARCHIVE     *
        ****************************************/
      
      /* check if a catalog is found */
      Job.catalog.get().isValid();
    }
    /* display filter, if defined */
    if (Job.config.get().filter != null) UserIO.print(UserIO.out.INFO,"Configured Regex filter: " + Job.config.get().filter.pattern());
    
    
    /**************************************************
      * (2) COUNT FILES/DIRECTORIES FROM EACH SNAPSHOT *
      **************************************************/
    UserIO.print(UserIO.out.DEBUG,"Indexing the archive...",false);
    /* total number of files/directories */
    int numSnapshotFiles[] = new int[Job.catalog.get().snapshotsIndex.size()];
    Arrays.fill(numSnapshotFiles, 0);
    
    /* number of new/modified files/directories */
    int numSnapshotFilesAdded[] = new int[Job.catalog.get().snapshotsIndex.size()];
    Arrays.fill(numSnapshotFilesAdded, 0);
    
    /* size (uncompressed) of files/directories */
    long sizeSnapshotFiles[] = new long[Job.catalog.get().snapshotsIndex.size()];
    Arrays.fill(sizeSnapshotFiles, 0);
    
    /* size (ncompressed) of new/modified files/directories */
    long sizeSnapshotFilesAdded[] = new long[Job.catalog.get().snapshotsIndex.size()];
    Arrays.fill(sizeSnapshotFilesAdded, 0);
    
    /* size (compressed) of files/directories */
    long sizeCompressedSnapshotFiles[] = new long[Job.catalog.get().snapshotsIndex.size()];
    Arrays.fill(sizeCompressedSnapshotFiles, 0);
    
    /* size (compressed) of new/modified files/directories */
    long sizeCompressedSnapshotFilesAdded[] = new long[Job.catalog.get().snapshotsIndex.size()];
    Arrays.fill(sizeCompressedSnapshotFilesAdded, 0);
    
    /* total size of the archived files */
    long sizeTotal = 0;
    
    /* count filtered files/directories from each snapshot */
    for (FileUnit u : Job.catalog.get().files) {
      if (u.isDeleted || u.isInvalid) continue;
      sizeTotal += u.compressedSize;
      if (u.matchesFilter()) {
        numSnapshotFilesAdded[u.snapshotsIndex.get(0)]++;
        sizeSnapshotFilesAdded[u.snapshotsIndex.get(0)] += u.size;
        sizeCompressedSnapshotFilesAdded[u.snapshotsIndex.get(0)] += u.compressedSize;
        for (int i : u.snapshotsIndex) {
          numSnapshotFiles[i]++;
          sizeSnapshotFiles[i] += u.size;
          sizeCompressedSnapshotFiles[i] += u.compressedSize;
        }
      }
    }
    /* count unique computers */
    HashMap<String,Integer> computers = new HashMap<String,Integer>();
    for (int i = 0; i < Job.catalog.get().snapshotsIndex.size(); i++) {
      if (! computers.containsKey(Job.catalog.get().snapshotsIndex.get(i).hostname)) 
        computers.put(Job.catalog.get().snapshotsIndex.get(i).hostname,new Integer(i));
    }

    Job.memoryUsage("numSnapshotFilesAdded",numSnapshotFilesAdded);
    Job.memoryUsage("sizeCompressedSnapshotFilesAdded",sizeCompressedSnapshotFilesAdded);
    Job.memoryUsage("numSnapshotFiles",numSnapshotFiles);
    Job.memoryUsage("numSnapshotFilesAdded",numSnapshotFilesAdded);
    Job.memoryUsage("sizeSnapshotFiles",sizeSnapshotFiles);
    UserIO.print(UserIO.out.DEBUG,"done");
    
    /************************************
      * (3) DISPLAY SNAPSHOT INFORMATION *
      ************************************/
    UserIO.print(UserIO.out.INFO,"Snapshots: " + Job.catalog.get().snapshotsIndex.size());
    UserIO.print(UserIO.out.INFO,"Computers: " + computers.size());
    UserIO.print(UserIO.out.INFO,"File revisions: " + Job.catalog.get().files.size());
    UserIO.print(UserIO.out.INFO,"Size: " + FileIO.getSize(sizeTotal));
    Job.gui(JobWrapper.msg.ARCHIVE_INFO,"File revisions: "+Job.catalog.get().files.size()+" - Size: "+FileIO.getSize(sizeTotal));
    
    UserIO.print(UserIO.out.INFO,Main.lineSeparator+" #        Computer        Timestamp          Snapshot             Total " +
                                 Main.lineSeparator+"----  ---------------  ---------------  ------------------  ------------------");
    
    for (int i = 0; i < Job.catalog.get().snapshotsIndex.size(); i++) {
      String encrypted = Job.catalog.get().snapshotsIndex.get(i).isEncrypted ? "(E)" : "";
      String deleted = Job.catalog.get().snapshotsIndex.get(i).isDeleted ? "(D)" : "";
      String number = (i+1)+"";
      String computer = Job.catalog.get().snapshotsIndex.get(i).hostname;
      String timestamp = Job.catalog.get().snapshotsIndex.get(i).time;
      String incremental = numSnapshotFilesAdded[i]+ " ("+FileIO.getSize(sizeCompressedSnapshotFilesAdded[i])+")";
      String full =  numSnapshotFiles[i]+" ("+FileIO.getSize(sizeSnapshotFiles[i])+")";
      
      UserIO.print(UserIO.out.INFO,"" +
                   UserIO.centerJustify(number+encrypted+deleted, 4) + "  " +
                   UserIO.centerJustify(computer, 15) + "  " +
                   UserIO.centerJustify(timestamp, 15) + "  " +
                   UserIO.centerJustify(incremental, 18) + "  " +
                   UserIO.centerJustify(full,18));
      @SuppressWarnings("deprecation")
	Object[] row = new Object[]{number,computer,new Boolean(Job.catalog.get().snapshotsIndex.get(i).isEncrypted),new Boolean(Job.catalog.get().snapshotsIndex.get(i).isDeleted),
    		  Job.catalog.get().snapshotsIndex.get(i).date.toLocaleString(),incremental,full};
      Job.gui(JobWrapper.msg.SNAPSHOT,row);
    }

    UserIO.print(UserIO.out.INFO,"");
    UserIO.print(UserIO.out.INFO,"(E): encrypted snapshot   (D): deleted snapshot");
    UserIO.print(UserIO.out.INFO,"");
  }
  
  /**
   * Display information of a given snapshot
   */
  public static void snapshotInfo() {
    UserIO.print(UserIO.out.INFO,"Requested action: SNAPSHOT INFO");
    /* check if a catalog is found */
    Job.catalog.get().isValid();
    /* display filter, if defined */
    if (Job.config.get().filter != null) UserIO.print(UserIO.out.INFO,"Configured Regex filter: " + Job.config.get().filter.pattern());
    if (! Job.catalog.get().hasSnapshot(Job.config.get().snapshot)) 
      UserIO.reportError("No snapshot #"+Job.config.get().snapshot+" has not been found in the catalog");
    if (Job.catalog.get().snapshotsIndex.get(Job.config.get().snapshot-1).isDeleted) 
      UserIO.reportError("The snapshot #"+Job.config.get().snapshot+" has been deleted");
    
    /* display specified snapshot number and timestamp */
    UserIO.print(UserIO.out.INFO,"Snapshot: " +Job.config.get().snapshot);
    
    /* retrieve the files belonging to this snapshot */
    final List<FileUnit> snapshotFiles = new ArrayList<FileUnit>();
    for (FileUnit u : Job.catalog.get().files) {
      /* matching the filter */
      if (u.snapshotsIndex.contains(Job.config.get().snapshot-1) && u.matchesFilter()) {
        snapshotFiles.add(u);
      }
    }
    Job.memoryUsage("snapshotFiles",snapshotFiles);
    
    /* print the files */
    UserIO.print(UserIO.out.INFO,"Total items: " + snapshotFiles.size());
    for (FileUnit u : snapshotFiles) {
      Timestamp timestamp = new Timestamp(u.time);
      Date date = new Date(timestamp.getTime());
      String dateString = Job.config.get().dateTimeFormatter.format(date);
      String sizeString = FileIO.getSize(u.size);
      String infoString = u.isDirectory ? dateString : sizeString+" - "+dateString;
      
      UserIO.print(UserIO.out.INFO," "+dateString+" ",false);
      if (u.isDirectory) UserIO.print(UserIO.out.INFO,"<DIR>  ",false);
      else UserIO.print(UserIO.out.INFO,"       ",false);
      
      UserIO.print(UserIO.out.INFO,u.nativeName,false);
      
      if (!u.isDirectory) UserIO.print(UserIO.out.INFO," ("+sizeString+")",false);
      UserIO.print(UserIO.out.INFO,"");
      
      Job.gui(JobWrapper.msg.SNAPSHOT_FILE, new String[] {"Snapshot #"+Job.config.get().snapshot,u.nativeName," ("+infoString+")"});
    }
    Job.gui(JobWrapper.msg.SNAPSHOT_FILE_READY,"");
  }
  
}