/**
 * TERMS AND CONDITIONS:
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.backup.zipsnap.ng.job.util;

import com.backup.zipsnap.ng.gui.JobWrapper;
import com.backup.zipsnap.ng.job.Job;
import com.backup.zipsnap.ng.job.util.FileIO;
import com.backup.zipsnap.ng.job.util.Timer;
import com.backup.zipsnap.ng.job.util.UserIO;
import com.backup.zipsnap.ng.Main;

/**
 * Implements a progress bar to keep track of the current status and ETA
 */
public class ProgressBar 
{
	private int progress = 0;
	private int previousProgress = 0;  
	private long previousSize = 0;
	private Timer timer = new Timer();
	private boolean firstRun = true;
	private String thisProgressBar = "progressBar";


	public ProgressBar()
	{
	}

	/**
	 * display the progress so far 
	 */
	public void display(long currentSize, long totalSize) {
		if (Main.settings.hideProgressDuringBatch && ! Job.config.get().interactive) return;
		/* first time it is run, start the timer, save the first size report */
		if (firstRun) {
			timer.start(thisProgressBar);
			previousSize = currentSize;
			firstRun = false;
		}
		/* calculate the percentage */
		progress = FileIO.percentage(currentSize,totalSize);
		if (progress != previousProgress) Job.gui(JobWrapper.msg.TASK_PROGRESS, progress);
		if ((int)progress/Main.settings.progressBarStep > previousProgress) {
			/* calculate the MB/s */
			long milliSeconds = timer.stopInMilliseconds(thisProgressBar);
			double seconds = (double)milliSeconds/1000; 
			double MBps = 0;
			double MB = 0;
			long ETA = 0;
			MB = (currentSize - previousSize)/1000000;
			if (seconds > 0) {
				MBps = MB/seconds;
				if (MBps > 0) ETA = (long) ((totalSize-currentSize)/1000000/MBps);
			}
			//UserIO.print(UserIO.out.INFO,"=> "+milliSeconds+" "+seconds+" "+currentSize+" "+previousSize+" "+MB+" "+MBps+" "+ETA);
			if (MB == 0 || ETA == 0) {
				UserIO.print(UserIO.out.INFO,progress+"%...",false);
			} else {
				UserIO.print(UserIO.out.INFO,progress+"% ("+ Timer.seconds2string(ETA)+" left)...",false);
				Job.gui(JobWrapper.msg.TASK_ETA, Timer.seconds2string(ETA));	
			}
			/* save the values for the next run */
			previousProgress = (int)progress/Main.settings.progressBarStep;
			previousSize = currentSize;
			timer.start(thisProgressBar);
		}

	}

}


